// ============================================================
// Project				:	CityRPG
// Author				:	Iban & JJstorm
// Description				:	Your new ScriptObject overlords.
// ============================================================
// Table of Contents
// 1. JobSO
// 2. CitySO
// 3. CalendarSO
// 4. ClothesSO
// 5. Stockmarket
// 6. PowerSO
// 7. WeatherSO
// 8. DaySO
// ============================================================

// ============================================================
// Section 1 : JobsSO
// ============================================================
function JobSO::populateJobs(%so)
{
	for(%a = 1; isObject(%so.job[%a]); %a++)
	{
		%so.job[%a].delete();
		%so.job[%a] = "";
	}
	
	// NOTE: Order is incredibly important. Jobs are referenced by ID, which is determined by order.
	// Mixing up the order of these professions will cause save data to reference the wrong job.
	%so.addJobFromFile("civilian");
	%so.addJobFromFile("grocer");
	%so.addJobFromFile("policeman");
	%so.addJobFromFile("criminal");
	%so.addJobFromFile("bountyhunter");
	%so.addJobFromFile("official");
	%so.addJobFromFile("miner");
	%so.addJobFromFile("shopkeeper");
	%so.addJobFromFile("armsdealer");
	%so.addJobFromFile("lumberjack");
	%so.addJobFromFile("laborer");
	%so.addJobFromFile("tailor");
	%so.addJobFromFile("mobster");
	%so.addJobFromFile("mobboss");
	%so.addJobFromFile("CEO");
	%so.addJobFromFile("ADMIN");
	
	// Export The Jobs
	%so.exportJob("civilian");
	%so.exportJob("grocer");
	%so.exportJob("policeman");
	%so.exportJob("criminal");
	%so.exportJob("bountyhunter");
	%so.exportJob("official");
	%so.exportJob("miner");
	%so.exportJob("shopkeeper");
	%so.exportJob("armsdealer");
	%so.exportJob("lumberjack");
	%so.exportJob("laborer");
	%so.exportJob("tailor");
	%so.exportJob("mobster");
	%so.exportJob("mobboss");
	%so.exportJob("CEO");
	%so.exportJob("ADMIN");
}

function JobSO::addJobFromFile(%so, %file)
{
	if(isFile("config/server/JJstorms CityRPG/Jobs/" @ %file @ ".cs"))
	{
		%so.addJob("config/server/JJstorms CityRPG/Jobs/" @ %file @ ".cs");
	}
	else
	{
		warn("No Such Job: " @ %file @ "at config/server/JJstorms CityRPG/Jobs/" @ %file @ ".cs Adding from default");

		if(isFile("./jobs/" @ %file @ ".cs"))
		{
			%so.addJob("./jobs/" @ %file @ ".cs");
		}
		else
		{
			warn("The default job " @ %file @ "was not located.(Add-Ons/Gamemode_JJstormsCityRPG/jobs/" @ %file @ ".cs)");
		}

	}
}
function JobSO::addJob(%so, %path) {
	%jobID = %so.getJobCount() + 1;
	exec(%path);
	%so.job[%jobID] = new scriptObject()
	{
		id		= $CityRPG::jobs::ID;
		
		name		= $CityRPG::jobs::name;
		invest		= $CityRPG::jobs::initialInvestment;
		pay		= $CityRPG::jobs::pay;
		tools		= $CityRPG::jobs::tools;
		education	= $CityRPG::jobs::education;
		db		= $CityRPG::jobs::datablock;
		hostonly	= $CityRPG::jobs::hostonly;
		adminonly	= $CityRPG::jobs::adminonly;
		outfit		= $CityRPG::jobs::outfit;
		
		sellItems	= $CityRPG::jobs::sellItems;
		sellFood	= $CityRPG::jobs::sellFood;
		sellServices 	= $CityRPG::jobs::sellServices; // Unused.
		sellClothes 	= $CityRPG::jobs::sellClothes;
		
		law		= $CityRPG::jobs::law;
		canPardon	= $CityRPG::jobs::canPardon;
		
		thief		= $CityRPG::jobs::thief;
		hideJobName	= $CityRPG::jobs::hideJobName;
		
		bountyOffer	= $CityRPG::jobs::offerer;
		bountyClaim	= $CityRPG::jobs::claimer;
		
		laborer		= $CityRPG::jobs::labor;
		
		tmHexColor	= $CityRPG::jobs::tmHexColor;
		helpline	= $CityRPG::jobs::helpline;
	};
	
	if(!isObject("CityRPGJob" @ %jobID @ "SpawnBrickData"))
	{
		datablock fxDtsBrickData(CityRPGSpawnBrickData : brickSpawnPointData)
		{
			category = "JJstorms CityRPG";
			subCategory = "JJstorms CityRPG Spawns";
			
			uiName = %so.job[%jobID].name SPC "Spawn";
			
			specialBrickType = "";
			
			CityRPGBrickType = 3;
			CityRPGBrickAdmin = true;
			
			spawnData = "jobSpawn" SPC %jobID;
		};
		
		CityRPGSpawnBrickData.setName("CityRPGJob" @ %jobID @ "SpawnBrickData");
	}
	
	deleteVariables("$CityRPG::jobs::*");
}


function JobSO::exportJob(%so, %file)
{
	if(isFile("./jobs/" @ %file @ ".cs"))
	{
		if(!isFile("config/server/JJstorms CityRPG/Jobs/" @ %file @ ".cs"))
		{
			GameTransferFile("./jobs/" @ %file @ ".cs", "config/server/JJstorms CityRPG/Jobs/" @ %file @ ".cs");
		}
		else
		{
			// warn("JobSO.exportJob: config/server/JJstorms CityRPG/Jobs/" @ %file @ ".cs already exists!");
		}
	}
	else
	{
		warn("JobSO.exportJob: ./jobs/" @ %file @ ".cs doesn't exists!");
	}
}

function JobSO::getAnAlias(%so)
{
	%jobCount = 0;
	
	for(%a = 1; isObject(%so.job[%a]); %a++)
	{
		if(!%so.job[%a].hideJobName)
		{
			%jobCount++;
			%jobName[%jobCount] = %so.job[%a].name;
		}
	}
	
	if(%jobCount)
		return %jobName[getRandom(1, %jobCount)];
	else
		return "JJstorm";
}

function JobSO::getJobCount(%so)
{
	for(%a = 0; isObject(%so.job[%a + 1]); %a++) { }
	return %a;
}

if(!isObject(JobSO))
{
	new scriptObject(JobSO) { };
	JobSO.populateJobs();
}

// ============================================================
// Section 2 : CitySO
// ============================================================
function CitySO::loadData(%so)
{
	if(isFile("config/server/JJstorms CityRPG/City.cs"))
	{
		exec("config/server/JJstorms CityRPG/City.cs");
		%so.minerals		= $CityRPG::temp::citydata::datumminerals;
		%so.lumber			= $CityRPG::temp::citydata::datumlumber;
	}
	else
	{
		%so.value["minerals"] = 0;
		%so.value["lumber"] = 0;
	}
}

function CitySO::saveData(%so)
{
	$CityRPG::temp::citydata::datum["minerals"]		= %so.minerals;
	$CityRPG::temp::citydata::datum["lumber"]		= %so.lumber;
	export("$CityRPG::temp::citydata::*", "config/server/JJstorms CityRPG/City.cs");
}

if(!isObject(CitySO))
{
	new scriptObject(CitySO) { };
	CitySO.loadData();
}

// ============================================================
// Section 3 : CalendarSO
// ============================================================
function CalendarSO::loadCalendar(%so)
{
	// Counters
	%so.numOfMonths = 12;
	%so.zbNumMonths = %so.numOfMonths - 1;
	
	// Names
	%so.nameOfMonth[1] = "January";
	%so.nameOfMonth[2] = "February";
	%so.nameOfMonth[3] = "March";
	%so.nameOfMonth[4] = "April";
	%so.nameOfMonth[5] = "May";
	%so.nameOfMonth[6] = "June";
	%so.nameOfMonth[7] = "July";
	%so.nameOfMonth[8] = "August";
	%so.nameOfMonth[9] = "September";
	%so.nameOfMonth[10] = "October";
	%so.nameOfMonth[11] = "November";
	%so.nameOfMonth[12] = "December";
	
	// Days
	%so.daysInMonth[0] = 31;
	%so.daysInMonth[1] = 28;
	%so.daysInMonth[2] = 31;
	%so.daysInMonth[3] = 30;
	%so.daysInMonth[4] = 31;
	%so.daysInMonth[5] = 30;
	%so.daysInMonth[6] = 31;
	%so.daysInMonth[7] = 31;
	%so.daysInMonth[8] = 30;
	%so.daysInMonth[9] = 31;
	%so.daysInMonth[10] = 30;
	%so.daysInMonth[11] = 31;
	
	for(%h = 0;%h < %so.getHolidayCount();%h++)
	{
		if($CityRPG::Calendar::Holiday[%h] !$= "" && $CityRPG::Calendar::HolidayDate[%h])
		{
				%so.holiday[$CityRPG::Calendar::HolidayDate[%h]] = $CityRPG::Calendar::Holiday[%h];
		}
	}
}
function CalendarSO::getHolidayCount(%so, %client)
{
	for(%h = 1; $CityRPG::Calendar::Holiday[%h] !$= ""; %h++)
	{
		%holidays++;
	}
	$CityRPG::Calendar::TotalHolidays = %holidays;
	return %holidays;
}

function CalendarSO::getDate(%so, %client)
{
	%day = %so.day;
	// If the tens place is 1, the suffix will always be "th"
	if(strlen(%day) > 1 && getSubStr(%day, (strlen(%day) - 2), 1) $= "1")
	{
		%suffix = "th";
	}
	// If not, it can either be "st," "nd," "rd," or "th," depending on the last numeral.
	else
	{
		switch(getSubStr(%day, (strlen(%day) - 1), 1))
		{
			case 1: %suffix = "st";
			case 2: %suffix = "nd";
			case 3: %suffix = "rd";
			default: %suffix = "th";
		}
	}
	
	messageAll('', '\c3[\c6%5 %6\c3] \c6Today, on \c3%1\c6 the \c3%2\c6%4 of year \c3%3\c6, ...', %so.nameOfMonth[%so.month], %so.day, %so.year, %suffix, $CityRPG::pref::radioname, $CityRPG::pref::radiofreq);
	
	if(%so.holiday[%so.month @ "." @ %so.day] !$= "")
		messageAll('', "\c6 - " @ %so.holiday[%so.month @ "." @ %so.day]);
		
	if(isObject(WeatherSO))
	{
		WeatherSO.todaysForecast();
	}
}

function CalendarSO::getMonth(%so)
{
	return %so.month;
}

function CalendarSO::dumpCalendar(%so)
{
	for(%a = 0; %so.daysInMonth[%a] !$= ""; %a++)
	{
		echo(%so.nameOfMonth[%a] SPC "has" SPC %so.daysInMonth[%a] SPC "days.");
	}
}

function CalendarSO::getYearLength(%so)
{
	for(%a = 0; %so.daysInMonth[%a] > 0; %a++)
	{
		%totalLength += %so.daysInMonth[%a];
	}
	
	return %totalLength;
}

function CalendarSO::getCurrentDay(%so)
{
	return (%so.date % %so.getYearLength());
}

function CalendarSO::loadData(%so)
{
	if(isFile("config/server/JJstorms CityRPG/Prefs.cs"))
	{
		exec("config/server/JJstorms CityRPG/Prefs.cs");
		%so.date = $CityRPG::temp::calendar::datumdate;
		%so.min = $CityRPG::temp::calendar::datummin;
		%so.hour = $CityRPG::temp::calendar::datumhour;
		%so.day = $CityRPG::temp::calendar::datumday;
		%so.month = $CityRPG::temp::calendar::datummonth;
		%so.year = $CityRPG::temp::calendar::datumyear;
	}
	else
	{
		%so.date = 1;
		%so.hour = 1;
		%so.min = 1;
		%so.month = 1;
		%so.day = 1;
	}
	
	%so.loadCalendar();
}

function CalendarSO::saveData(%so)
{
	$CityRPG::temp::calendar::datumdate	= %so.date;
	$CityRPG::temp::calendar::datummin	= %so.min;
	$CityRPG::temp::calendar::datumhour	= %so.hour;
	$CityRPG::temp::calendar::datumday	= %so.day;
	$CityRPG::temp::calendar::datummonth	= %so.month;
	$CityRPG::temp::calendar::datumyear	= %so.year;
	
	export("$CityRPG::temp::calendar::*", "config/server/JJstorms CityRPG/Calendar.cs");
}

if(!isObject(CalendarSO))
{
	new scriptObject(CalendarSO) { };
	CalendarSO.schedule(1, "loadData");
}

// ============================================================
// Section 4 : ClothesSO
// ============================================================
function ClothesSO::loadClothes(%so)
{
	// Clothing Data
	%so.color["none"]		= "1 1 1 1";
	%so.node["none"]		= "0";
	
	// Outfits
	// Outfits use index instead of names.
	// Do not repeat indexes.
	// This is the order they appear in the GUI.
	%so.str[1]	= "none none none none whitet whitet skin bluejeans blackshoes default default";
	%so.uiName[1]	= "Default";
	%so.sellName[1]	= "Default Suit";

	%so.str[2]	= "none brownhat keep keep greenshirt greenshirt skin greyPants blackshoes default default";
	%so.uiName[2]	= "Basic";
	%so.sellName[2]	= "Basic Outfit";
	
	%so.str[3]	= "keep skullcap keep keep blackshirt blackshirt blackgloves blackPants blackshoes default default";
	%so.uiName[3]	= "Gimp";
	%so.sellName[3]	= "Gimp Suit";
	
	%so.str[4]	= "none none none none whitet redsleeve keep brightbluePants blueshoes default default";
	%so.uiName[4]	= "Blockhead";
	%so.sellName[4]	= "Blockhead Clothes";

	%so.str[5]	= "keep keep keep keep greenshirt greenshirt keep brownPants blackshoes default worm-sweater";
	%so.uiName[5]	= "Nerd";
	%so.sellName[5]	= "Nerd Suit";

	%so.str[6]	= "keep keep keep keep blackshirt blackshirt keep blackPants blackshoes default Mod-Suit";
	%so.uiName[6]	= "Business";
	%so.sellName[6]	= "Business Suit";

	%so.str[7]	= "keep keep keep keep blueshirt blueshirt keep bluePants blackshoes default Mod-Suit";
	%so.uiName[7]	= "Council";
	%so.sellName[7]	= "Council Suit";

	%so.str[8]	= "keep keep keep keep skingen skingen skingen skingen skingen default default";
	%so.uiName[8]	= "Naked";
	%so.sellName[8]	= "B-Day Suit";

	%so.str[9]	= "none brownhat none none blackshirt blackshirt skin bluejeans blackshoes default default";
	%so.uiName[9]	= "Gangsta";
	%so.sellName[9]	= "Gangsta";

	%so.str[10]	= "custom custom custom custom custom custom custom custom custom custom custom";
	%so.uiName[10]	= "Custom_Clothes";
	%so.sellName[10]	= "Custom Clothes";
	
	%so.str[11]	= "keep keep keep keep keep keep blackgloves keep keep keep keep";
	%so.uiName[11]	= "Black_Gloves";
	%so.sellName[11]	= "Black Gloves";
	
	%so.str[12]	= "keep brownhat keep keep keep keep keep keep keep keep keep";
	%so.uiName[12]	= "Brown_Hat";
	%so.sellName[12]	= "Brown Hat";
	
	%so.str[13]	= "keep piratehat keep keep keep keep keep keep keep keep keep";
	%so.uiName[13]	= "Pirate_Hat";
	%so.sellName[13]	= "Pirate Hat";
	
	%so.str[14]	= "keep cophat keep keep keep keep keep keep keep keep keep";
	%so.uiName[14]	= "Cop_Hat";
	%so.sellName[14]	= "Cop Hat";
	
	%so.str[15]	= "keep skullcap keep keep keep keep keep keep keep keep keep";
	%so.uiName[15]	= "Skull_Cap";
	%so.sellName[15]	= "Skull Cap";

	// Hats
	%so.color["brownhat"]	= "0.329 0.196 0.000 1.000";
	%so.node["brownhat"]	= "4";
	%so.str["brownhat"]	= "keep this keep keep keep keep keep keep keep";
	
	%so.color["piratehat"]	= "0.078 0.078 0.078 1";
	%so.node["piratehat"]	= "5";
	%so.str["piratehat"]	= "keep this keep keep keep keep keep keep keep";
	
	%so.color["copHat"]	= "0 0.141176 0.333333 1";
	%so.node["copHat"]	= "6";
	%so.str["copHat"]	= "keep this keep keep keep keep keep keep keep";
	
	%so.color["skullcap"]	= "0.200 0.200 0.200 1.000";
	%so.node["skullcap"]	= "7";
	%so.str["skullcap"]	= "keep this keep keep keep keep keep keep keep";

	// Extra Wear	
	%so.color["cape"]	= "1.000 0.200 0.000 1.000";
	%so.node["cape"]	= "7";
	%so.str["cape"]	= "keep this keep keep keep keep keep keep keep";
	
	// Gloves
	%so.color["blackgloves"] = "0.200 0.200 0.200 1.000";
	%so.node["blackgloves"]	= "0";
	%so.str["blackgloves"]	= "keep keep keep keep keep keep this keep keep";
	
	// Shirts
	%so.color["pinkt"]	= "1 0.75 0.79 1";
	%so.node["pinkt"]	= "gender";
	%so.str["pinkt"]	= "keep keep keep keep this this keep keep keep";
	
	%so.color["pinktank"]	= "1 0.07 0.57 1";
	%so.node["pinktank"]	= "gender";
	%so.str["pinktank"]	= "keep keep keep keep this skingen keep keep keep";
	
	%so.color["whitet"]	= "1 1 1 1";
	%so.node["whitet"]	= "gender";
	%so.str["whitet"]	= "keep keep keep keep this this keep keep keep";
	
	%so.color["copShirt"]	= "0 0.141176 0.333333 1";
	%so.node["copShirt"]	= "gender";
	%so.str["copShirt"]	= "keep keep keep keep this this keep keep keep";
	
	%so.color["jumpsuit"]	= "1 0.617 0 1";
	%so.node["jumpsuit"]	= "gender";
	%so.str["jumpsuit"]	= "keep keep keep keep this this keep this this";
	
	%so.color["blackshirt"]	= "0.200 0.200 0.200 1.000";
	%so.node["blackshirt"]	= "gender";
	%so.str["blackshirt"]	= "keep keep keep keep this this keep keep keep";
	
	%so.color["brownshirt"]	= "0.329 0.196 0.000 1.000";
	%so.node["brownshirt"]	= "gender";
	%so.str["brownshirt"]	= "keep keep keep keep this this keep keep keep";
	
	%so.color["greenshirt"]	= "0.00 0.262 0.00 1";
	%so.node["greenshirt"]	= "gender";
	%so.str["greenshirt"]	= "keep keep keep keep this this keep keep keep";
	
	%so.color["blueshirt"]	= "0.0 0.141 0.333 1";
	%so.node["blueshirt"]	= "gender";
	%so.str["blueshirt"]	= "keep keep keep keep this this keep keep keep";
	
	%so.color["orangeshirt"]= "1 0.200 0 1";
	%so.node["orangeshirt"]	= "gender";
	%so.str["orangeshirt"]	= "keep keep keep keep this this keep keep keep";

	// Pants
	%so.color["bluejeans"]	= "0 0.141 0.333 1";
	%so.node["bluejeans"]	= "0";
	%so.str["bluejeans"]	= "keep keep keep keep keep keep keep this keep";
	
	%so.color["blackPants"] = "0.200 0.200 0.200 1.000";
	%so.node["blackPants"]	= "0";
	%so.str["blackPants"]	= "keep keep keep keep keep keep keep this keep";
	
	%so.color["brownPants"] = "0.329 0.196 0.000 1.000";
	%so.node["brownPants"]	= "0";
	%so.str["brownPants"]	= "keep keep keep keep keep keep keep this keep";
	
	%so.color["greyPants"]  = "0.300 0.300 0.300 1.000";
	%so.node["greyPants"]	= "0";
	%so.str["greyPants"]	= "keep keep keep keep keep keep keep this keep";

	%so.color["brightbluePants"] = "0.200 0.000 0.800 1.000";
	%so.node["brightbluePants"]	= "0";
	%so.str["brightbluePants"]	= "keep keep keep keep keep keep keep this keep";

	%so.color["bluePants"] = "0.0 0.141 0.333 1";
	%so.node["bluePants"]	= "0";
	%so.str["bluePants"]	= "keep keep keep keep keep keep keep this keep";

	// Shoes
	%so.color["blackshoes"]	= "0.200 0.200 0.200 1.000";
	%so.node["blackshoes"]	= "0";
	%so.str["blackshoes"]	= "keep keep keep keep keep keep keep keep this";

	%so.color["brownshoes"]	= "0.329 0.196 0.000 1.000";
	%so.node["brownshoes"]	= "0";
	%so.str["brownshoes"]	= "keep keep keep keep keep keep keep keep this";

	%so.color["blueshoes"]	= "0.200 0.000 0.800 1.000";
	%so.node["blueshoes"]	= "0";
	%so.str["bluehoes"]	= "keep keep keep keep keep keep keep keep this";

	//Misc
	%so.color["redsleeve"] = "0.900 0.220 0.000 1.000";
	%so.node["redsleeve"] = "gender";
	%so.color["redsleeve"] = "keep keep keep keep this this keep keep keep";
	
	%so.color["orangesleeve"] = "1.000 0.200 0.000 1.000";
	%so.node["orangesleeve"] = "gender";
	%so.color["orangesleeve"] = "keep keep keep keep this this keep keep keep";
}

function ClothesSO::postEvents(%so)
{
	%str = "list";
	
	for(%a = 1; %so.str[%a] !$= ""; %a++)
		%str = %str SPC %so.uiName[%a] SPC %a;
	
	if(%str !$= "")
	{
		registerOutputEvent("fxDTSBrick", "sellClothes", %str TAB "int 1 500 1");
		
		for(%b = 0; %b < ClientGroup.getCount(); %b++)
		{
			%subClient = ClientGroup.getObject(%b);
			serverCmdRequestEventTables(%subClient);
			messageClient(%subClient, '', "\c6Your Event Tables have been updated. If you do not know what that means, ignore this message.");
		}
	}
}

function ClothesSO::getColor(%so, %client, %item)
{
	if(%item $= "skin" || %item $= "skingen")
		return %client.headColor;
	else
	{
		%color = %so.color[%item];
		return %color;
	}
}

function ClothesSO::getNode(%so, %client, %item)
{
	if(%item $= "skin")
		return 0;
	else if(%item $= "skingen")
		return (%client.gender $= "Male" ? 0 : 1);
	else
	{
		%node = %so.node[%item];
	
		if(%node $= "gender" || %node $= "skingen")
			return (%client.gender $= "Male" ? 0 : 1);
		else
			return %node;
	}
}

function ClothesSO::getDecal(%so, %client, %segment, %item)
{
	if(%item $= "" || %item $= "default")
	{
		if(%segment $= "face")
			return "smiley";
		else if(%segment $= "chest")
			return "AAA-none";
	}
	else
		return %item;
}

function ClothesSO::giveItem(%so, %client, %item)
{
	if(strLen(%so.str[%item]) && isObject(%client))
	{
		%outfit = CityRPGData.getData(%client.bl_id).valueOutfit;
		
		for(%a = 0; %a < getWordCount(%outfit); %a++)
		{
			if(getWord(%so.str[%item], %a) $= "keep")
				%newOutfit = (%newOutfit $= "" ? getWord(%outfit, %a) : %newOutfit SPC getWord(%outfit, %a));
			else if(getWord(%so.str[%item], %a) $= "this")
				%newOutfit = (%newOutfit $= "" ? %item : %newOutfit SPC %item);
			else if(getWord(%so.str[%item], %a) $= "custom")
				%newOutfit = (%newOutfit $= "" ? custom : %newOutfit SPC custom);
			else
				%newOutfit = (%newOutfit $= "" ? getWord(%so.str[%item], %a) : %newOutfit SPC getWord(%so.str[%item], %a));
		}
		
		CityRPGData.getData(%client.bl_id).valueOutfit = %newOutfit;
		%client.applyBodyParts();
		%client.applyBodyColors();
	}
}

if(!isObject(ClothesSO))
{
	new scriptObject(ClothesSO) { };
	ClothesSO.schedule(1, "loadClothes");
	ClothesSO.schedule(1, "postEvents");
}

// ============================================================
// Section 5 : StockMarket
// ============================================================

function StockMarket::Funds(%so)
{
	for(%cl=0;%cl < clientgroup.getCount(); %cl++)
	{
		%client = clientgroup.getObject(%cl);
		if($CityRPG::StockMarket::ClientHasToBeOnline)
		{
			if(%client.hasSpawnedOnce)
			{
				%business = CityRPGData.getData(%client.bl_id).valuebussiness;
				if(%business !$= "")
				{
					CityRPGData.getData(%client.bl_id).valuebank += StockMarket.getBusinessFunds(%business);
				}
			}
		}
		else
		{
			%business = CityRPGData.getData(%client.bl_id).valuebussiness;
			if(%business !$= "")
			{
				CityRPGData.getData(%client.bl_id).valuebank += mCeil(StockMarket.getBusinessFunds(%business));
			}	
		}
	}
}

function Stockmarket::getBusinessFunds(%so, %business)
{
	for(%cl=0;%cl < clientgroup.getCount(); %cl++)
	{
		%client = clientgroup.getObject(%cl);
		%business = CityRPGData.getData(%client.bl_id).valuebusiness;
		if(%business !$= "")
		{
			if(StockMarket.stock[%business] !$= "" && StockMarket.Funds.stock[%business] !$= "")
			{
				return StockMarket.Funds.stock[%business];
			}
			else
			{
				StockMarket.stock[%business] = %business;
				StockMarket.Funds.stock[%business] = getrandom(5, -5) @ "." @ getrandom(0, 9);
				StockMarket.getBusinessFunds(%business);
			}
		}
		
	}
}

function StockMarket::tick(%so)
{
	%isBusiness = 0;
	StockMarket.Funds();
	for(%cl=0;%cl < clientgroup.getCount(); %cl++)
	{
		%client = clientgroup.getObject(%cl);
		%business = CityRPGData.getData(%client.bl_id).valuebusiness;
		%funds = StockMarket.Funds.stock[%business];
		if(%business !$= "" && %funds !$= "")
		{
			%isBusiness = 1;
			if(%funds > 0) { %color = "<color:00ff00>"; } else { %color = "<color:ff0000>"; }
			%display = %display SPC %business @ "\c6: " @ %color @ %funds @ "\c6 | \c3";
		}
	}
	if(%isBusiness) { messageAll('', '\c6 - Todays StockMarket %1', %display); }
}

if(!isObject(Stockmarket))
{
	new ScriptObject(StockMarket) { };
	StockMarket.tick();
}

// ============================================================
// Section 6 : PowerSO
// ============================================================

function PowerSO::tick(%so, %client)
{
	if(isObject(%client))
	{
		if(%client.isAdmin && $CityRPG::pref::PowerAdminProof == 1) { return; }
		
		if(CityRPGData.getData(%client.bl_id).valuepower == 0)
		{
			if(!%client.LightsAreOn)
			{
				return;
			}
			
			for(%a=0;%a < %client.brickgroup.getcount();%a++)
			{
				%brick = %client.brickgroup.getObject(%a);
				%lightID = %brick.getlightID();
				if(%lightID != "-1")
				{
					%light = %lightID.getDatablock();
					if(%lightID != "-1")
					{
						%so.Light[%brick] = %light;
						%so.Light.Brick[%brick] = %brick;
						%so.Light.Brick.Owner[%brick] = %client;
						%brick.setlight();
						%client.hasLights = 1;
						echo("Brick: " @ %brick @ "(" @ %so.Light.Brick.Owner[%brick].name @ ") light was set to NONE");
					}
					else if(CityRPGData.getData(%client.bl_id).valuepower == 1)
					{
						%brick.setlight(%so.Light[%brick]);
					}
				}
			}
			messageClient(%client, '', '\c6 - Your lights have been shut off because you havent paid your electric bill');
			%client.LightsAreOn = 0;
		}
		else
		{
			if(%client.hasLights && !%client.LightsAreOn)
			{
				for(%a=0;%a < %client.brickgroup.getcount();%a++)
				{
					%brick = %client.brickgroup.getObject(%a);
					if(isObject(%brick))
					{
						if(%brick.getLightID() != "-1")
						{
							%so.Light[%brick] = %brick.getlightID().getdatablock();
							%so.Light.Brick[%brick] = %brick;
							%so.Light.Brick.Owner[%brick] = %client;
						}
						%brick.setLight(%so.Light[%brick]);
					}
				}
				messageClient(%client, '', '\c6 - Your lights have been turned back on.');
				%client.LightsAreOn = 1;
			}
			else
			{
				for(%a=0;%a < %client.brickgroup.getcount();%a++)
				{
					%brick = %client.brickgroup.getObject(%a);
					if(isObject(%brick))
					{
						if(%brick.getLightID() != "-1")
						{
							%so.Light[%brick] = %brick.getlightID().getdatablock();
							%so.Light.Brick[%brick] = %brick;
							%so.Light.Brick.Owner[%brick] = %client;
							%client.hasLights = 1;
							%client.LightsAreOn = 1;
						}
					}
				}
			}
		}
	}
}

function PowerSO::TurnLightsOn(%so, %client)
{
	if(isObject(%client))
	{
		if(!%client.LightsAreOn)
		{
			if(%client.brickgroup.getcount() > 0)
			{
				for(%a=0;%a < %client.brickgroup.getcount();%a++)
				{
					%brick = %client.brickgroup.getobject(%a);
					if(%so.Light[%brick] !$= "")
					{
						%brick.setLight(%so.Light[%brick]);
					}
				}
			}
		}
	}
}

function PowerSO::getBricks(%number)
{
	if($BrickCount != mainBrickGroup.getcount())
	{
		for(%a=0;%a<mainBrickGroup.getcount();%a++)
		{
			%brick = mainBrickGroup.getObject(%a);
			PowerSO.Brick[%a] = %brick;
			if(%brick.getLightID() !$= "-1")
			{
				PowerSO.Light[%brick] = %brick.getLightID();
			}
			$BrickCount = %a;
		}
	}
	else if(%number !$= "")
	{
		return PowerSO.Brick[%number];
	}
}

if(!isObject(PowerSO))
{
	new ScriptObject(PowerSO) { };
}

// ============================================================
// Section 7 : WeatherSO
// ============================================================
function WeatherSO::loadForecast(%so)
{
	// Rain Probability
	%so.rainProb[1] = $CityRPG::enviro::RainMonth1; //"January";
	%so.rainProb[2] = $CityRPG::enviro::RainMonth2; //"February";
	%so.rainProb[3] = $CityRPG::enviro::RainMonth3; //"March";
	%so.rainProb[4] = $CityRPG::enviro::RainMonth4; //"April";
	%so.rainProb[5] = $CityRPG::enviro::RainMonth5; //"May";
	%so.rainProb[6] = $CityRPG::enviro::RainMonth6; //"June";
	%so.rainProb[7] = $CityRPG::enviro::RainMonth7; //"July";
	%so.rainProb[8] = $CityRPG::enviro::RainMonth8; //"August";
	%so.rainProb[9] = $CityRPG::enviro::RainMonth9; //"September";
	%so.rainProb[10] = $CityRPG::enviro::RainMonth10; //"October";
	%so.rainProb[11] = $CityRPG::enviro::RainMonth11; //"November";
	%so.rainProb[12] = $CityRPG::enviro::RainMonth12; //"December";
	
	// Snow Probability
	%so.snowProb[1] = $CityRPG::enviro::SnowMonth1; //"January";
	%so.snowProb[2] = $CityRPG::enviro::SnowMonth2; //"February";
	%so.snowProb[3] = $CityRPG::enviro::SnowMonth3; //"March";
	%so.snowProb[4] = $CityRPG::enviro::SnowMonth4; //"April";
	%so.snowProb[5] = $CityRPG::enviro::SnowMonth5; //"May";
	%so.snowProb[6] = $CityRPG::enviro::SnowMonth6; //"June";
	%so.snowProb[7] = $CityRPG::enviro::SnowMonth7; //"July";
	%so.snowProb[8] = $CityRPG::enviro::SnowMonth8; //"August";
	%so.snowProb[9] = $CityRPG::enviro::SnowMonth9; //"September";
	%so.snowProb[10] = $CityRPG::enviro::SnowMonth10; //"October";
	%so.snowProb[11] = $CityRPG::enviro::SnowMonth11; //"November";
	%so.snowProb[12] = $CityRPG::enviro::SnowMonth12; //"December";
}

function WeatherSO::todaysForecast(%so)
{
	%month = CalendarSO.month;
	
	%snowProb = %so.snowProb[%month];
	%rainProb = %so.rainProb[%month];
	%clearProb = 100 - (%so.snowProb[%month] + %so.rainProb[%month]);
	
	if(%snowProb > 0)
		%snowPow = getRandom(0, %snowProb);
	if(%rainProb > 0)
		%rainPow = getRandom(0, %rainProb);
	if(%clearProb > 0)
		%clearPow = getRandom(0, %clearProb);
	
	if(%clearProb == 100)
		%forecast = "clear";
	else if(%snowPow > %rainPow && %snowPow > %clearPow)
		%forecast = "snow";
	else if(%rainPow > %snowPow && %rainPow > %clearPow)
		%forecast = "rain";
	else
		%forecast = "";
	
	if(%forecast $= "") { %fforecast = "clear"; } else { %fforecast = %forcast; }
	$CityRPG::enviro::CurrWeather = %fforecast;
	
	if(%forecast !$= "")
	{
		%intensity = getRandom(0, 50);
		
		if(%forecast $= $CityRPG::enviro::CurrWeather) {
			%verb = "remains";
		}
		else
		{
			$CityRPG::enviro::CurrWeather = %forecast;
			%verb = "is";
		}
		
		%intensity = getRandom(5, 15);
		
		switch$(%forecast)
		{
			case "snow":
				%so.generateWeather("snow", %intensity);
				messageAll('', '\c6 - Today\'s forecast %1 Snowy.', %verb);
			case "rain":
				%so.generateWeather("rain", %intensity);
				messageAll('', '\c6 - Today\'s forecast %1 Rainy.', %verb);
			default:
				%so.generateWeather("clear", 0);
				messageAll('', '\c6 - Today\'s forecast %1 \c0INVALID\c6.', %verb);
		}
	}
	else
	{
		messageAll('', "\c6 - Today's forecast is Clear.");
		%so.generateWeather("clear", 0);
	}
}

function WeatherSO::generateWeather(%so, %weather, %intensity)
{
	// Crash Fix
	// if(%weather !$= "snow" || %weather !$= "rain") { %weather = "clear"; }
	
	$CityRPG::enviro::CurrWeather = %weather;
	
	if(%weather $= "clear")
	{
		if(isObject(Precipitation))
			Precipitation.delete();
		
		if(CityRPGData.Soundloop)
			cancel(CityRPGData.Soundloop);
		
		if(mFloor(%intensity) >= 1)
		{
			%extra = " WITH A FOG INTENSITY OF " @ %intensity @ ".";
			%r = getword($Sky.SkySolidColor, 0);
			%g = getword($Sky.SkySolidColor, 1);
			%b = getword($Sky.SkySolidColor, 2);
			%fr = getword($Sky.SkySolidColor, 0);
			%fg = getword($Sky.SkySolidColor, 1);
			%fb = getword($Sky.SkySolidColor, 2);
			$CityRPG::enviro::FogIntensity = %intensity;
			DaySO.setsky(%r,%g,%b,%fr,%fg,%fb,%intensity);
		}
		else
		{
			%extra = ".";
			DaySO.getday();
		}
		warn("WEATHER ADJUSTING TO:" SPC "[" @ %weather @ "]" @ %extra);
		return;
	}
	
	if(isObject(Precipitation))
		Precipitation.delete();
	
	if(%weather !$= "" && (%intensity = mFloor(%intensity)) > 0)
	{		
		switch$(%weather)
		{
			case "snow":
				%db = "SnowA";
				%mutliplier = 0.15;
			case "rain":
				%db = "CityRPG_Rain";
				%mutliplier = 0.15;
			default:
				%db = "SnowA";
				%mutliplier = 0.15;
		}
		
		%intensity = %intensity * %mutliplier;
		// Default Properties
		%numDrops = 400 * %intensity;
		%minSpeed = 0.5 * %intensity;
		%maxSpeed = 1 * %intensity;
		%minMass = 0.75 * %intensity;
		%maxMass = 0.85 * %intensity;
		%maxTurbulence = 0.1 * %intensity;
		%turbulenceSpeed = 0.5 * %intensity;
		
		new Precipitation(Precipitation)
		{
			dataBlock = %db;
			numDrops = %numDrops;
			
			position = "191.844 621.068 566.484";
			rotation = "1 0 0 0";
			scale = "1 1 1";
			doCollision = "1";
			
			boxWidth = "100";
			boxHeight = "100";
			
			minSpeed = %minSpeed;
			maxSpeed = %maxSpeed;
			
			minMass = %minMass;
			maxMass = %maxMass;
			
			useTurbulence = "false";
			maxTurbulence = %maxTurbulence;
			turbulenceSpeed = %turbulenceSpeed;
			rotateWithCamVel = "false";
		};
		
		if(CityRPGData.Soundloop)
			cancel(CityRPGData.Soundloop);
		
		switch$(%weather)
		{
			case "rain":
					%loop = (($CityRPG::tick::speed * 60000) * (24 - CalendarSO.hour) / 16000);
					PlayLoopingSound("RainSound", %loop, 16);
					DaySO.getday();
			case "snow":
					%loop = (($CityRPG::tick::speed * 60000) * (24 - CalendarSO.hour) / 8000);
					PlayLoopingSound("SnowSound", %loop, 8);
					DaySO.getday();
			
		}
	}
	else
	{
		DaySO.getday();
	}
}

if(!isObject(WeatherSO))
{
	new scriptObject(WeatherSO) { };
	WeatherSO.loadForecast();

}

// ============================================================
// Section 8 : DaySO
// ============================================================
function DaySO::getday(%so)
{
	if($CityRPG::enviro::CurrWeather $= "rain" || $CityRPG::enviro::CurrWeather $= "snow")
	{
		if(!isObject(Precipitation))
		{
			%intensity = getrandom(1, 50);
			WeatherSO.generateweather($CityRPG::enviro::CurrWeather, %intensity);
		}
	}
	else
	{
		if($CityRPG::enviro::CurrWeather $= "clear" && $CityRPG::enviro::FogIntensity > 0)
		{
			WeatherSO.generateweather($CityRPG::enviro::CurrWeather, $CityRPG::enviro::FogIntensity);
		}
	}
	
	%timehour = CalendarSO.hour;
	DaySO.generateday(%timehour);
	for(%a=0;%a<clientGroup.getCount();%a++)
	{
		%client = clientgroup.getobject(%a);
		if(isObject(%client))
		{
			commandtoclient(%client, 'lightscene');
		}
	}
}

function DaySO::setsun(%so,%r,%g,%b,%scale)
{
	if(isObject(Sun))
		Sun.delete();
	
	%pos = "10.0045 -444.123" SPC (20.5 * CalendarSO.hour);
	
	$Sun = new Sun(Sun)
	{
		color = %r SPC %g SPC %b SPC "1";
		ambient = %r SPC %g SPC %b SPC "1";
		scale = %scale SPC %scale SPC %scale;
		locked = "false";
		direction = "0.57735 0.57735 -0.57735";
		position = %pos;
		canSaveDynamicFields = "1";
		rotation = "1 0 0 0";
	};
}

function DaySO::setSunFX(%so,%path,%r,%g,%b,%time,%scale)
{
	%tickele = $CityRPG::tick::speed * 1207;
	
	if(isObject(Sunfx))
		Sunfx.delete();

	$Sunfx = new fxSunLight(SunFX)
	{
		position = "121.979 -83.8798 51.8515";
		rotation = "0 0 0 0";
		scale = %scale SPC %scale SPC %scale;
		Enable = "1";
		Locked = "true";
		LocalFlareBitmap = "None";
		RemoteFlareBitmap = %path;
		SunAzimuth = "100";
		SunElevation = "30";
		LockToRealSun = "1";
		FlareTP = "1";
		Colour = %r SPC %g SPC %b SPC "1.000000";
		Brightness = "1";
		FlareSize = "0.5";
		FadeTime = "0.1";
		BlendMode = "0";
		AnimColour = "0";
		AnimBrightness = "0";
		AnimRotation = "0";
		AnimSize = "0";
		AnimAzimuth = "0";
		AnimElevation = "1";
		LerpColour = "1";
		LerpBrightness = "1";
		LerpRotation = "1";
		LerpSize = "1";
		LerpAzimuth = "1";
		LerpElevation = "1";
		LinkFlareSize = "0";
		SingleColourKeys = "1";
		MinColour = "0.000000 0.000000 0.000000 1.000000";
		MaxColour = %r SPC %g SPC %b SPC "1.000000";
		MinBrightness = "0";
		MaxBrightness = "1";
		MinRotation = "0";
		MaxRotation = "359";
		minSize = "0.5";
		maxSize = "1";
		MinAzimuth = "0";
		MaxAzimuth = "359";
		MinElevation = "-30";
		MaxElevation = "210";
		RedKeys = "AZA";
		GreenKeys = "AZA";
		BlueKeys = "AZA";
		BrightnessKeys = "AZA";
		RotationKeys = "AZA";
		SizeKeys = "AZA";
		AzimuthKeys = "AZ";
		ElevationKeys = "AZ";
		ColourTime = %time;
		BrightnessTime = %time;
		RotationTime = %time;
		SizeTime = %time;
		AzimuthTime = %time;
		ElevationTime = %tickele;
	};
}
function DaySO::setSky(%so,%r,%g,%b,%fr,%fg,%fb,%fog)
{
	if(isObject($Sky))
		$Sky.delete();
	
	if($CityRPG::enviro::CurrWeather $= "rain")
	{
		%VD = getrandom(10, 100);
	}
	else if($CityRPG::enviro::CurrWeather $= "snow")
	{
		%VD = getrandom(10, 200);
	}
	else if($CityRPG::enviro::CurrWeather $= "clear")
	{
		%VD = %fog;
	}
	%FD = mFloor(%VD/2);
	
	%timehour = CalendarSO.hour;
	if(%timehour >= 20 || %timehour == 1) { %extra = "_Night"; %usesky = "1";} else { %extra = ""; %usesky = "0";}
	%path = "Add-Ons/Client_JJstormsCityRPG/shapes/Skies/" @ $CityRPG::enviro::CurrWeather @ %extra @ ".dml";
	
	if(isObject(Sky)) { Sky.delete(); }
	$Sky = new Sky(Sky)
	{
		position = "336 136 0";
		rotation = "1 0 0 0";
		scale = "1 1 1";
		materialList = %path;
		cloudHeightPer[0] = "0.349971";
		cloudHeightPer[1] = "0.3";
		cloudHeightPer[2] = "0.199973";
		cloudSpeed1 = "0.0005";
		cloudSpeed2 = "0.001";
		cloudSpeed3 = "0.0003";
		visibleDistance = %VD;
		fogDistance = %FD;
		fogColor = %fr SPC %fg SPC %fb SPC "1.000000";
		fogStorm1 = "0";
		fogStorm2 = "0";
		fogStorm3 = "0";
		fogVolume1 = "0 0 0";
		fogVolume2 = "0 0 0";
		fogVolume3 = "0 0 0";
		fogVolumeColor1 = "128.000000 128.000000 128.000000 128.000000";
		fogVolumeColor2 = "128.000000 128.000000 128.000000 128.000000";
		fogVolumeColor3 = "128.000000 128.000000 128.000000 128.000000";
		windVelocity = "1 1 0";
		windEffectPrecipitation = "1";
		SkySolidColor = %r SPC %g SPC %b SPC "1.000000";
		useSkyTextures = %usesky;
		renderBottomTexture = "0";
		noRenderBans = "0";
	};
}
function DaySO::generateday(%so, %timehour)
{	
	if(%timehour > 12) { %fog = (25 - %timehour) * 100; } else { %fog = %timehour * 100; }
	
	switch$(%timehour)
	{
		case 1:
			DaySO.setSun(0.1, 0.1, 0.1, 1);
			DaySO.setSky(0, 0, 0, 0.1, 0.1, 0.1, %fog);
		case 2:
			DaySO.setSunFX("None",1,1,1,5,1);
			DaySO.setSun(0.25, 0.25, 0.25, 1);
			DaySO.setSky(0.25, 0.25, 0.25, 0.15, 0.15, 0.15, %fog);
		case 3:
			DaySO.setSun(0.4, 0.4, 0.5, 1);
			DaySO.setSky(0.4, 0.4, 0.5, 0.3, 0.3, 0.3, %fog);
		case 4:
			DaySO.setSun(0.5, 0.5, 0.6, 1);
			DaySO.setSky(0.5, 0.5, 0.6, 0.4, 0.4, 0.4, %fog);
		case 5:
			DaySO.setSun(0.55, 0.55, 0.65, 1);
			DaySO.setSky(0.55, 0.55, 0.65, 0.45, 0.45, 0.45, %fog);
		case 6:
			DaySO.setSun(0.6, 0.6, 0.7, 1);
			DaySO.setSky(0.6, 0.6, 0.7, 0.5, 0.5, 0.5, %fog);
		case 7:
			DaySO.setSun(0.65, 0.65, 0.75, 1);
			DaySO.setSky(0.65, 0.65, 0.75, 0.55, 0.55, 0.55, %fog);
		case 8:
			DaySO.setSun(0.7, 0.7, 0.8, 1);
			DaySO.setSky(0.7, 0.7, 0.8, 0.6, 0.6, 0.6, %fog);
		case 9:
			DaySO.setSun(0.75, 0.75, 0.85, 1);
			DaySO.setSky(0.75, 0.75, 0.85, 0.7, 0.7, 0.7, %fog);
		case 10:
			DaySO.setSun(0.8, 0.8, 0.9, 1);
			DaySO.setSky(0.8, 0.8, 0.9, 0.7, 0.7, 0.7, %fog);
		case 11:
			DaySO.setSun(0.85, 0.85, 0.95, 1);
			DaySO.setSky(0.85, 0.85, 0.95, 0.8, 0.8, 0.8, %fog);
		case 12:
			DaySO.setSun(0.9, 0.9, 1, 1);
			DaySO.setSky(0.9, 0.9, 1, 0.9, 0.9, 0.9, %fog);
		case 13:
			DaySO.setSun(0.85, 0.8, 0.8, 1);
			DaySO.setSky(0.85, 0.8, 0.8, 0.7, 0.7, 0.7, %fog);
		case 14:
			DaySO.setSun(0.8, 0.7, 0.7, 1);
			DaySO.setSky(0.8, 0.7, 0.7, 0.7, 0.7, 0.7, %fog);
		case 15:
			DaySO.setSun(0.8, 0.6, 0.6, 1);
			DaySO.setSky(0.8, 0.6, 0.6, 0.6, 0.6, 0.6, %fog);
		case 16:
			DaySO.setSun(0.75, 0.5, 0.5, 1);
			DaySO.setSky(0.75, 0.5, 0.5, 0.5, 0.5, 0.5, %fog);
		case 17:
			DaySO.setSun(0.7, 0.4, 0.3, 1);
			DaySO.setSky(0.7, 0.4, 0.3, 0.4, 0.4, 0.4, %fog);
		case 18:
			DaySO.setSun(0.5, 0.3, 0.3, 1);
			DaySO.setSky(0.5, 0.3, 0.3, 0.3, 0.3, 0.3, %fog);
		case 19:
			DaySO.setSun(0.2, 0.2, 0.3, 1);
			DaySO.setSky(0.2, 0.2, 0.3, 0.2, 0.2, 0.2, %fog);
		case 20:
			DaySO.setSunFX("Add-Ons/Client_JJstormsCityRPG/shapes/Skies/Moon",1,1,1,5,2.5);
			DaySO.setSun(0.1, 0.1, 0.3, 1);
			DaySO.setSky(0, 0, 0, 0.1, 0.1, 0.2, %fog);
		case 21:
			DaySO.setSun(0.1, 0.1, 0.2, 1);
			DaySO.setSky(0, 0, 0, 0, 0, 0.2, %fog);
		case 22:
			DaySO.setSun(0.05, 0.05, 0.2, 1);
			DaySO.setSky(0, 0, 0, 0, 0, 0.2, %fog);
		case 23:
			DaySO.setSun(0.01, 0.01, 0.2, 1);
			DaySO.setSky(0, 0, 0, 0, 0, 0.2, %fog);
		case 24:
			DaySO.setSun(0, 0, 0.1, 1);
			DaySO.setSky(0, 0, 0, 0, 0, 0.0, %fog);
	}
}
if(!isObject(DaySO))
{
	new scriptObject(DaySO) { };
	%so.day = $CityRPG_Temp::CurrDay;
}
